!------------------------------------------------------------------------------------!
! David Julian Kling | 2016 | Johann Wolfgang Goethe-Universität Frankfurt am Main   !
!------------------------------------------------------------------------------------!



module params
implicit none
      integer, parameter :: kra = selected_real_kind(16) ! Genauigkeitsparameter
      integer, parameter :: len_string_num = 5 ! Stellen des Zahlenfeldes in Ausgabedateien
      character(len_string_num), parameter :: len_string = '5' ! Stellen des Zahlenfeldes als String 

      integer, parameter :: unit_M_R_rel = 50 ! zum Schreiben in Dateien für Masse-Radius-Beziehung
      integer, parameter :: unit_single  = 51 ! zum Schreiben in Datei für bestimmte zentrale Dichte
      integer, parameter :: unit_test    = 52 ! zum Ausgeben in Datei für Tests

      real(kra), parameter :: zero  = 0.0_kra
      real(kra), parameter :: one   = 1.0_kra
      real(kra), parameter :: two   = 2.0_kra
      real(kra), parameter :: three = 3.0_kra
      real(kra), parameter :: four  = 4.0_kra
      real(kra), parameter :: five  = 5.0_kra
      real(kra), parameter :: six   = 6.0_kra
      real(kra), parameter :: nine  = 9.0_kra
      real(kra), parameter :: ten   = 10.0_kra

      real(kra), parameter :: pi  = 4.0_kra*atan(1.0_kra)
      real(kra), parameter :: pi2 = pi*pi


!----------------------------------!
! Naturkonstanten in cgs-Einheiten !
!----------------------------------!
      real(kra), parameter :: G    = 6.6740831_kra*10.0_kra**(-8.0_kra)    ! [cm^3/g/s^2]
      real(kra), parameter :: c    = 29979245800.0_kra                     ! [cm/s] 
      real(kra), parameter :: c2   = c*c                                   ! c^2
      real(kra), parameter :: c4   = c2*c2                                 ! c^4
      real(kra), parameter :: hbar = 1.054571800_kra*10.0_kra**(-27.0_kra) ! [erg s]
      real(kra), parameter :: m_e  = 9.1093835611*10.0_kra**(-28.0_kra)    ! Elektronen-Masse [g]
      real(kra), parameter :: m_n  = 1.674927471214*10.0_kra**(-24.0_kra)  ! Neutronen-Masse [g]
!----------------------------------!

!--------------------------------------------------------------------------------!
! Umrechnungsfaktoren von geometrischen Einheiten (G=C=M_sun=1) -> cgs-Einheiten !
!--------------------------------------------------------------------------------!
! (wenn Ergebnisse in geometrischen Einheiten gewünscht werden, sind die    !
! folgenden Umrechnungsfaktoren =1 zu setzten; die Umrechnung erfolgt erst  !
! unmittelbar vor dem Schreiben in die Datei. Allerdings müsse in dem Fall, !
! dass die Ausgabe in geometrischen Einheiten gewünscht ist, die Werte      !
! der vordefinierten Polytropen-Konstanten entsprechend angepasst werden.   !
!---------------------------------------------------------------------------!
       real(kra), parameter :: m_geom_to_cgs   = c2/G !1.9884_kra  *ten**33.0_kra ! Masse
       real(kra), parameter :: rho_geom_to_cgs = c2/G !6.17714_kra *ten**17.0_kra ! Ruhemassen-Dichte
       real(kra), parameter :: P_geom_to_cgs   = c4/G !5.55173_kra *ten**38.0_kra ! Druck
       real(kra), parameter :: K_e_non_rel_geom_to_cgs   = P_geom_to_cgs / rho_geom_to_cgs**(five/three) ! polytrope Konstante, e-gas, nicht-rel.
       real(kra), parameter :: K_e_rel_geom_to_cgs       = P_geom_to_cgs / rho_geom_to_cgs**(four/three) ! polytrope Konstante, e-gas, rel.
       real(kra), parameter :: K_e_ultra_rel_geom_to_cgs = P_geom_to_cgs / rho_geom_to_cgs ! polytrope Konstante, e-gas, ultra-rel.
       real(kra), parameter :: K_n_non_rel_geom_to_cgs   = K_e_non_rel_geom_to_cgs        ! polytrope Konstante, purer Neutronenstern, nicht-rel.
!-------------------------------------------------------------------------!


!-----------------------------!
! Charakteristische Parameter !
!-----------------------------!
!!!!!! Vorisicht:
        ! Die Konvertierungs-Faktoren sind alle so definiert, dass sie von geometrischen in
        ! cgs-Einheiten konvertieren, da wir aber in diesem Abschnitt von cgs nach geometrischen
        ! Einheiten umrechnen müssen wir teilen, anstatt zu multiplizieren.
!!!!!!



      real(kra), parameter :: Z_over_A = one/two   ! := 1/eta, Z_over_A = 2 macht Sinn, wenn man annimmt,
                                                   ! dass ein WZ vornehmlich aus C- und O-Kernen besteht. 

      real(kra), parameter :: gamma_poly_e_non_rel = five/three
      real(kra), parameter :: K_e_non_rel   = hbar*hbar/(15.0_kra*pi2*m_e) * (three*pi2*Z_over_A/m_n)**(five/three) &
                                            & / K_e_non_rel_geom_to_cgs ! e-Gas, nicht-rel.
      real(kra), parameter :: gamma_poly_e_rel = four/three
      real(kra), parameter :: K_e_rel       = hbar*c/(12.0_kra*pi2) * (three*pi2*Z_over_A/m_n)**(four/three) & 
                                            & / K_e_rel_geom_to_cgs ! e-Gas, rel.
      real(kra), parameter :: gamma_poly_e_ultra_rel = one
      real(kra), parameter :: K_e_ultra_rel = c2/(three) &
                                            & / K_e_ultra_rel_geom_to_cgs ! e-Gas, ultra-rel.
      real(kra), parameter :: gamma_poly_n_non_rel = five/three
      real(kra), parameter :: K_n_non_rel   = hbar*hbar/(15.0_kra*pi2*m_n) * (three*pi2/m_n)**(five/three) &
                                            & / K_n_non_rel_geom_to_cgs ! purer Neutronenstern, nicht-rel.
      real(kra), parameter :: K          = K_n_non_rel ! polytrope Konstante    
      real(kra), parameter :: gamma_poly = gamma_poly_n_non_rel
!-----------------------------!

!----------------------!
! Numerische Parameter !
!----------------------!
     ! real(kra), parameter :: stepsize_init = 100000.0  ! Weiße Zwerge 
      real(kra), parameter :: stepsize_init = 1000.0     ! Neutronenstern
!----------------------!


!---------------------!
! Anfangsbedingungen: !
!---------------------!
      real(kra), parameter :: r_init = zero
      real(kra), parameter :: m_init = zero      
    !-------------------------------------------------------------------------!
    ! Bereich der zentralen Drücke, für die TOV-Gleichung gelöst werden soll: !
    !-------------------------------------------------------------------------!
     ! real(kra), parameter :: P_init = 0.5_kra*ten**(22.0_kra) / P_geom_to_cgs ! nicht-rel. e-Gas
     ! real(kra), parameter :: P_fin  = 4.5_kra*ten**(22.0_kra) / P_geom_to_cgs !      -"- 
     ! real(kra), parameter :: P_init = 0.6_kra*ten**(24.0_kra) / P_geom_to_cgs ! rel. e-Gas
     ! real(kra), parameter :: P_fin  = 4.0_kra*ten**(25.0_kra) / P_geom_to_cgs !      -"- 
      real(kra), parameter :: P_init = 2.0_kra*ten**(31.0_kra) / P_geom_to_cgs ! nicht-rel. NS
     ! real(kra), parameter :: P_fin  = 3.0_kra*ten**(33.0_kra) / P_geom_to_cgs ! -"- P(r)-M(r)-Diagramm
      real(kra), parameter :: P_fin  = 3.0_kra*ten**(41.0_kra) / P_geom_to_cgs ! -"- M-R-Relation
    !-----------------------------------------------------------------------------------------!
    ! Falls nur für einen charackt. Wert (=> P_fin egal) gelöst werden soll (Beispiel-Werte): !
    !-----------------------------------------------------------------------------------------!
     ! real(kra), parameter :: P_init = 2.33002_kra*ten**(22.0_kra) / P_geom_to_cgs ! nicht-rel. e-Gas
     !! real(kra), parameter :: P_init = 5.619_kra*ten**(25.0_kra) / P_geom_to_cgs ! rel. e-Gas
     !! real(kra), parameter :: P_init = 5.0_kra*ten**(32.0_kra) / P_geom_to_cgs ! nicht-rel. NS
     ! real(kra), parameter :: P_fin  = P_init ! nicht-rel./rel./ultra-rel. e-Gas und nicht-rel. NS 
!---------------------!

      real(kra), parameter :: r_max = 1000000000000000000.0_kra ! Falls keine Konvergenz, stoppt bei diesem Radius

!--------------------------------------!
! Parameter für Masse-Radius-Beziehung !
!--------------------------------------!
      integer, parameter :: points_M_R = 100 ! Anzahl der verschiedenen Dichten, für die die
                    ! TOV-Gleichung gelöst werden, soll, bzw. Anzahl der der Punkte im M-R-Diagramm
!--------------------------------------!

end module params





